from collections.abc import Iterable
from typing import Final, Self

from edge import eventlog
from edge.support import fs
from edge.types.exceptions import *
from edge.types.mmr import MMR_CLASSES
from edge.webserver.sysconf.types.conf import IUiConfProvider
from edge.webserver.sysconf.types.defs import *
from edge.webserver.sysconf.uiconf.builder import CTRL_DEF_UICONF_DIR, UconfBuilder
from edge.webserver.sysconf.uiconf.terminal import TerminalUiconf
from edge.webserver.sysconf.uiconf.uiconf2 import UiConf2
from edge.webserver.sysconf.uimod import rootinfo
from edge.webserver.sysconf.uimod import unet
from edge.webserver.sysconf.uimod.eventstore import EsUimod
from edge.webserver.sysconf.uimod.uref import Uref, Uref2
from edge.webserver.sysconf.uimod.urole import *
from edge.webserver.sysconf.uimod.utype import *


def get_spool_dir(ui_pid: str) -> str:
    return fs.join_path(CTRL_SPOOL_DIR, f'pipeline-{ui_pid}')


def build_uconf(unit_utype: str, unit_uref: str, factory_name: str, factory_conf: dict | None = None) -> dict:
    factory_info = unet.decode_factory(fs.join_path(CTRL_DEF_UICONF_DIR, factory_name))
    uconf_builder = UconfBuilder(factory_info.defconf)
    return uconf_builder.build(unit_utype, unit_uref, factory_conf)


def build_term_unit_uconf(term_uref: str, unit_utype: str, unit_uref: str, factory_conf: dict | None = None) -> dict:
    term_umod = Umod.create(term_uref)
    term_uconf = TerminalUiconf(term_umod.conf_path)
    return build_uconf(unit_utype, unit_uref, term_uconf.factory_name, factory_conf)


class Umod(Uref, IUiConfProvider):
    def __init__(self, uimod_pid: str, uimod_type: str, uimod_id: str) -> None:
        super().__init__(uimod_pid, uimod_type, uimod_id)
        self._conf = None

    def __hash__(self) -> int:
        return hash(self.type + self.id)

    def __str__(self) -> str:
        return self.ref

    def __eq__(self, other: Self) -> bool:
        return (isinstance(other, self.__class__) and
                getattr(other, 'pid', None) == self.pid and
                getattr(other, 'type', None) == self.type and
                getattr(other, 'id', None) == self.id)

    @classmethod
    def create(cls, uref: str) -> Self:
        """
        :param uref:
            'wrong-way.1_14'
        """
        uiref: Uref = cls.create_uref(uref)
        return cls(uiref.pid, uiref.type, uiref.id)

    @classmethod
    def create_term(cls, term_pid: str) -> Self:
        uiref: Uref = cls.create_term_uref(term_pid)
        return cls(uiref.pid, uiref.type, uiref.id)

    @property
    def conf_dir(self) -> str:
        return fs.join_path(TERM_UICONF_DIR, self.pid, self.type)

    @property
    def conf_path(self) -> str:
        old_pid, old_id = divmod(int(self.id), 10)
        return fs.join_path(self.conf_dir, f'{old_id}.json')
        # return fs.join_path(self.conf_dir, '{}.json'.format(self.id))

    def load_conf(self, **kwargs) -> dict:
        """
        :raise FsError:
        """
        if self._conf is None:
            self._conf = fs.load_json3(self.conf_path, **kwargs)

        return self._conf

    def save_conf(self, conf_jnode: dict) -> None:
        fs.write_json_file(conf_jnode, self.conf_path, pretty=True)

    @property
    def schema_path(self) -> str:
        old_pid, old_id = divmod(int(self.id), 10)
        return fs.join_path(TERM_UISCHEMA_DIR, self.pid, self.type, f'{old_id}.json')

    @property
    def schema_limits(self) -> dict | None:
        schema_jnode = fs.load_json3(self.schema_path)
        return schema_jnode.get('xLimits')

    @property
    def lock_path(self) -> str:
        # TODO: migrate to /run/lock/edge/
        return fs.join_path('/var/run/edge', '{}.lock').format(self.ref)

    def create_backup_dirpath(self) -> str:
        import time
        backup_action = '{:d}-{}'.format(int(round(time.time() * 1e6)), self.ref)
        backup_dir = fs.join_path(CTRL_BACKUP_DIR, backup_action)
        return backup_dir

    def install_schema(self) -> None:
        conf_path: Final[str] = self.conf_path
        if fs.path_exists(conf_path):
            fs.ln_schema(self.schema_path, conf_path)

    def install(self, uconf: dict) -> None:
        assert self.type not in UTYPES_FILELESS

        fs.silent_mkdir_r(self.conf_dir)
        fs.write_json_file(uconf, self.conf_path, pretty=True)
        self.install_schema()

    def uninstall(self) -> None:
        if self.type not in UTYPES_FILELESS:
            fs.silent_rm_r(self.conf_path)
            fs.silent_rm_r(self.schema_path)


class Umod2(Uref2, IUiConfProvider):
    def __init__(self, ref: str) -> None:
        super().__init__(ref)
        self._conf_jnode: dict | None = None
        self._container_path: str | None = None

    def __eq__(self, other: Self) -> bool:
        return isinstance(other, self.__class__) and getattr(other, 'ref', None) == self.ref

    def __hash__(self) -> int:
        return hash(self.ref)

    def __str__(self) -> str:
        return self.ref

    @property
    def pid(self) -> str:
        eventlog.log_e('fail: unsupported')
        raise NotImplementedError()

    @property
    def id(self) -> str:
        eventlog.log_e('fail: unsupported')
        raise NotImplementedError()

    @property
    def lock_path(self) -> str:
        if self.type in UTYPES_FILELESS:
            # TODO: migrate to /run/lock/edge/
            return fs.join_path('/var/run/edge', '{}.lock').format(self.ref)

        return self.uiconf.lock_path

    def create_backup_dirpath(self) -> str:
        import time
        backup_action = '{:d}-{}'.format(int(round(time.time() * 1e6)), self.ref)
        backup_dir = fs.join_path(CTRL_BACKUP_DIR, backup_action)
        return backup_dir

    def assert_ref(self) -> None:
        """
        :raises NotFoundError:
        """
        self.load_conf()

    @property
    def parent(self) -> Self:
        """
        :raises NotFoundError:
        """
        return Umod2(self.parent_ref)

    def child(self, child_type: str, child_chan: str | int) -> Self:
        return Umod2(self.child_ref(child_type, child_chan))

    def load_conf(self) -> dict:
        """
        :raise NotFoundError:
        """
        if None in [self._container_path, self._conf_jnode]:
            container_names_desc = []
            segments = self.ref.split('~')
            for index in range(0, len(segments)):
                container_name = '~'.join(segments[0:index + 1]) + '.json'
                container_names_desc.append(container_name)

            for container_name in reversed(container_names_desc):
                container_path = fs.join_path(CTRL_UICONF_DIR, container_name)
                container_jnode = fs.load_json3(container_path, default=None) or dict()
                conf_jnode = container_jnode.get(self.ref)
                if conf_jnode:
                    self._container_path = container_path
                    self._conf_jnode = conf_jnode
                    return conf_jnode

            raise NotFoundError

        return self._conf_jnode

    @property
    def schema_path(self) -> str:
        eventlog.log_e('fail: unsupported')
        raise NotImplementedError()

    @property
    def conf_path(self) -> str:
        import warnings
        warnings.warn('message', PendingDeprecationWarning, stacklevel=2)

        self.assert_ref()
        return self._container_path

    def save_conf(self, conf_jnode: dict) -> None:
        self.assert_ref()
        container_jnode = fs.load_json3(self._container_path, default=None) or dict()
        container_jnode[self.ref] = conf_jnode
        fs.write_json_file(container_jnode, self._container_path, pretty=True)

        self._conf_jnode = conf_jnode

    @property
    def uiconf(self) -> UiConf2:
        """
        :raise NotFoundError:
        """
        return UiConf2(self.load_conf())

    def save_uiconf(self, conf: UiConf2) -> None:
        """
        :rtype: None
        """
        with open(SEMVER_PATH, 'r') as f:
            ver = f.readlines()[0].strip()

        conf.set_ver(ver)
        self.save_conf(conf.raw)

    def install(self, conf: UiConf2, container_ref: str | None = None) -> None:
        if self.type in UTYPES_FILELESS:
            return

        try:
            self.assert_ref()
            eventlog.log_e('fail: InternalError (already installed)')
            raise InternalError

        except NotFoundError:
            pass

        if not self.ref.startswith(container_ref):
            raise NotImplementedError

        container_name = f'{container_ref or self.ref}.json'
        container_path = fs.join_path(CTRL_UICONF_DIR, container_name)
        container_jnode = fs.load_json3(container_path, default=None) or dict()
        container_jnode[self.ref] = conf.raw
        fs.write_json_file(container_jnode, container_path, pretty=True)

        self._container_path = container_path
        self._conf_jnode = conf.raw

    def uninstall(self) -> None:
        if self.type in UTYPES_FILELESS:
            return

        self.assert_ref()
        container_jnode = fs.load_json3(self._container_path, default=None) or dict()
        del container_jnode[self.ref]
        if len(container_jnode) > 0:
            fs.write_json_file(container_jnode, self._container_path, pretty=True)
        else:
            fs.silent_rm_r(self._container_path)


# noinspection PyClassHasNoInit
class RootInfoUiconf:
    @classmethod
    def get_term_uinfo(cls, term_uuid: str) -> unet.UnitInfo | None:
        root_uinfo = rootinfo.decode_info().root
        terms_uinfo: list[unet.UnitInfo] = unet.map_units_r(root_uinfo, utypes=[UTYPE_TERMINAL])

        for term_uinfo in terms_uinfo:
            term_uimod = Umod.create(term_uinfo['ref'])
            term_uiconf = term_uimod.load_conf()
            if term_uuid == term_uiconf['ro']['uuid']:
                return term_uinfo

    @classmethod
    def get_term_pid(cls, term_uuid: str) -> str | None:
        term_uinfo = cls.get_term_uinfo(term_uuid)
        if term_uinfo:
            term_uimod = Umod.create(term_uinfo['ref'])
            return term_uimod.pid

    @classmethod
    def get_term_event_types(cls, term_uinfo: unet.UnitInfo) -> list[str]:
        det_types = [
            UTYPE_DET_TRANSIT,
            UTYPE_DET_PARKING,
            UTYPE_DET_WRONG_WAY,
            UTYPE_DET_REDLIGHT,
            UTYPE_DET_WANTED,
            UTYPE_DET_SECTION_SPEED,
            UTYPE_DET_BUS_LANE,
            UTYPE_DET_SEATBELT
        ]

        if unet.find_unit_r(term_uinfo, utype=UTYPE_RADAR):
            det_types.append(UTYPE_DET_SPOT_SPEED)

        return det_types

    @classmethod
    def get_raterm_pid(cls) -> str | None:
        root_uinfo = rootinfo.decode_info().root
        radar_uinfo = unet.find_unit_r(root_uinfo, utype=UTYPE_RADAR)
        if radar_uinfo:
            return Umod.create(radar_uinfo['ref']).pid

    @classmethod
    def get_uimods(cls, root: unet.UnitInfo, urefs: Iterable[str]) -> set[IUiConfProvider]:
        uimods = set()
        for uref in urefs:
            uinfo = unet.find_unit_r(root, uref=uref)
            if uinfo is None:
                eventlog.log_w(f'fail: find_unit_r (uiref:{uref})')
                continue

            utype = uinfo['type']
            if utype == UTYPE_WEBSERVER:
                uimod = Umod2(uinfo['ref'])

            elif utype == UTYPE_EVENTSTORE:
                uimod = EsUimod.create(uinfo['ref'])

            elif utype in UTYPES_UIMOD2:
                uimod = Umod2(uinfo['ref'])

            else:
                uimod = Umod.create(uinfo['ref'])

            uimods.add(uimod)

        return uimods

    @classmethod
    def add_term(cls, ctrl_uref: str,
                 factory_uinfo: unet.UnitInfo,
                 factory_links: list[unet.UnetLink]) -> tuple[unet.UnitInfo, unet.UnitInfo]:
        """
        decode_term_factory(ctrl_uinfo, factory_name) -> tuple[term_uinfo, term_links, term_conf]
            - ctrl_uinfo is const
            - term_uinfo contains UnitInfos with remapped urefs
            - term_links contains UnitLinks with remmaped urefs
            - term_conf contains remmaped uref_to_defconf_map

        :returns: tuple[root_uinfo, term_uinfo]
            [0] contains the root_uinfo with term_uinfo added (with remapped urefs)
            [1] contains the term_uinfo (with remapped urefs)
        """

        unet_info = rootinfo.decode_info()

        # create a uref map
        uref_gen = rootinfo.UrefGenerator(unet_info.root)
        term_uref = uref_gen.create_term_uref().ref

        uref_gen = rootinfo.UrefGenerator(unet_info.root)
        old_to_new_factory_urefs: dict[str, str] = uref_gen.create_uref_map(factory_uinfo, term_uref)

        # remap urefs in the links
        for factory_link in factory_links:
            factory_link[0] = old_to_new_factory_urefs.get(factory_link[0], factory_link[0])
            factory_link[1] = old_to_new_factory_urefs.get(factory_link[1], factory_link[1])

        unet_info.links.extend(factory_links)

        # remap urefs in the infos
        def replace_uref_fn(uinfo: unet.UnitInfo) -> None:
            uinfo['ref'] = old_to_new_factory_urefs[uinfo['ref']]

        unet.map_units_r(factory_uinfo, replace_uref_fn)

        # update the root-info
        ctrl_uinfo = unet.find_unit_r(unet_info.root, uref=ctrl_uref)
        ctrl_uinfo['units'].append(factory_uinfo)

        def find_link(links_list: list[unet.UnetLink], ref1: str, ref2: str) -> bool:
            for link in links_list:
                if ref1 in link and ref2 in link:
                    return True

            return False

        tr_urefs = unet.find_urefs_r(unet_info.root, utypes=[UTYPE_TRAFFIC_RULES])
        iit_ca_urefs = unet.find_urefs_r(unet_info.root, utypes=[UTYPE_IIT_CA])
        iit_ca_eu_urefs = unet.find_urefs_r(unet_info.root, utypes=[UTYPE_IIT_CA_END_USER])

        for tr_uref in tr_urefs:
            for iit_ca_uref in iit_ca_urefs:
                if not find_link(unet_info.links, tr_uref, iit_ca_uref):
                    unet_info.links.append((tr_uref, iit_ca_uref))

            for iit_ca_eu_uref in iit_ca_eu_urefs:
                if not find_link(unet_info.links, tr_uref, iit_ca_eu_uref):
                    unet_info.links.append((tr_uref, iit_ca_eu_uref))

        rootinfo.encode_info(unet_info)

        return unet_info.root, factory_uinfo

    @classmethod
    def add_term_detector(cls, term_uref: str, det_utype: str, det_name: str) -> tuple[Uref, unet.UnitInfo]:
        root_unet = rootinfo.decode_info()
        root_info, root_links = root_unet.root, root_unet.links

        term_uinfo = unet.find_unit_r(root_info, uref=term_uref)
        if term_uinfo is None:
            eventlog.log_d(f'fail: NotFoundError ({UTYPE_TERMINAL})')
            raise NotFoundError

        rules_uinfo = unet.find_unit_r(term_uinfo, utype=UTYPE_TRAFFIC_RULES)
        if rules_uinfo is None:
            eventlog.log_d(f'fail: NotFoundError ({UTYPE_TRAFFIC_RULES})')
            raise NotFoundError

        uref_gen = rootinfo.UrefGenerator(rules_uinfo)
        det_uref: Uref = uref_gen.create_unit_uref(term_uref, det_utype)

        rules_uinfo['units'].append({
            'ref': det_uref.ref,
            'type': det_uref.type,
            'name': det_name,
            'role': UROLE_DETECTOR,
            'internal': False
        })

        if det_utype == UTYPE_DET_SPOT_SPEED:
            hstreaming_uinfo = unet.find_unit_r(term_uinfo, utype=UTYPE_HTTP_STREAMING)
            if hstreaming_uinfo is None:
                eventlog.log_d(f'fail: NotFoundError ({UTYPE_HTTP_STREAMING})')
            else:
                root_links.append((det_uref.ref, hstreaming_uinfo['ref']))

            rstreaming_uinfo = unet.find_unit_r(term_uinfo, utype=UTYPE_RTSP_STREAMING)
            if rstreaming_uinfo is None:
                eventlog.log_d(f'fail: NotFoundError ({UTYPE_RTSP_STREAMING})')
            else:
                root_links.append((det_uref.ref, rstreaming_uinfo['ref']))

        elif det_utype in [UTYPE_DET_REDLIGHT, UTYPE_DET_RAILROAD]:
            hstreaming_uinfo = unet.find_unit_r(term_uinfo, utype=UTYPE_HTTP_STREAMING)
            if hstreaming_uinfo is None:
                eventlog.log_d(f'fail: NotFoundError ({UTYPE_HTTP_STREAMING})')
            else:
                root_links.append((det_uref.ref, hstreaming_uinfo['ref']))

            rstreaming_uinfo = unet.find_unit_r(term_uinfo, utype=UTYPE_RTSP_STREAMING)
            if rstreaming_uinfo is None:
                eventlog.log_d(f'fail: NotFoundError ({UTYPE_RTSP_STREAMING})')
            else:
                root_links.append((det_uref.ref, rstreaming_uinfo['ref']))

            traffic_light_uinfo = unet.find_unit_r(term_uinfo, utype=UTYPE_TRAFFIC_LIGHT)
            if traffic_light_uinfo is None:
                eventlog.log_d(f'fail: NotFoundError ({UTYPE_TRAFFIC_LIGHT})')
            else:
                root_links.append((det_uref.ref, traffic_light_uinfo['ref']))

            rtsp_cam_uinfo = unet.find_unit_r(term_uinfo, utype=UTYPE_RTSP_CAMERA)
            if rtsp_cam_uinfo is None:
                eventlog.log_d(f'fail: NotFoundError ({UTYPE_RTSP_CAMERA})')
            else:
                root_links.append((det_uref.ref, rtsp_cam_uinfo['ref']))

            mv_cam_uinfo = unet.find_unit_r(term_uinfo, utype=UTYPE_MV_CAMERA)
            if mv_cam_uinfo is None:
                eventlog.log_d(f'fail: NotFoundError ({UTYPE_MV_CAMERA})')
            else:
                root_links.append((det_uref.ref, mv_cam_uinfo['ref']))

        root_links.append((det_uref.ref, rules_uinfo['ref']))
        rootinfo.encode_info(root_unet)
        return det_uref, root_info

    @classmethod
    def add_term_service(cls,
                         term_uref: str,
                         service_utype: str,
                         service_name: str,
                         utypes_to_link: list[str]) -> tuple[Uref, unet.UnitInfo]:

        unet_info = rootinfo.decode_info()
        term_info = unet.find_unit_r(unet_info.root, uref=term_uref)
        if term_info is None:
            eventlog.log_d(f'fail: NotFoundError ({UTYPE_TERMINAL})')
            raise NotFoundError

        uref_gen = rootinfo.UrefGenerator(term_info)
        service_uref: Uref = uref_gen.create_unit_uref(term_uref, service_utype)

        if term_info.get('units') is None:
            term_info['units'] = []

        term_info['units'].append({
            'ref': service_uref.ref,
            'type': service_uref.type,
            'name': service_name,
            'role': UROLE_SERVICE,
            'internal': False
        })

        for type_to_link in utypes_to_link:
            unit_to_link_info = unet.find_unit_r(term_info, utype=type_to_link)
            unet_info.links.append((service_uref.ref, unit_to_link_info['ref']))

        rootinfo.encode_info(unet_info)
        return service_uref, unet_info.root


# noinspection PyClassHasNoInit
class CamDgStats:
    def __init__(self, jnode_path: str):
        self._jnode = fs.load_json3(jnode_path, default=None)

    @classmethod
    def create(cls, term_pid: str) -> Self:
        ui_id = '0'
        jnode_path = fs.join_path(f'/var/spool/edge/pipeline-{term_pid}/zabbix/vs{ui_id}')
        return cls(jnode_path)

    def _steam_active(self) -> bool:
        stats_jnode = self._jnode
        if stats_jnode:
            stream_jnode = stats_jnode.get('stream')
            if stream_jnode is not None:
                # old format
                return stream_jnode['error'] == 0
            else:
                units_jnode: list[dict] = stats_jnode.get('units') or list()
                bus_streamers = [x for x in units_jnode if x.get('type') == 'bus-streamer']
                max_endpoints = 0
                max_jnode = dict()
                for bus_streamer in bus_streamers:
                    stat_jnode = bus_streamer.get('stat') or dict()
                    if stat_jnode.get('endpoints', 0) > max_endpoints:
                        max_jnode = stat_jnode

                return max_jnode.get('active', False)

        return False

    def test_availability(self) -> None:
        cam_status = self._steam_active()
        if not cam_status:
            eventlog.log_w('fail: CamServiceError (cam-status:<none>)')
            raise CamServiceError

    def get_stream_wh(self) -> tuple[int, int] | None:

        stats_jnode = self._jnode
        if stats_jnode:
            stream_jnode = stats_jnode.get('stream')
            if stream_jnode is not None:
                # old format
                return stream_jnode.get('wh')
            else:
                units_jnode: list[dict] = stats_jnode.get('units') or list()
                bus_streamers = [x for x in units_jnode if x.get('type') == 'bus-streamer']
                max_endpoints = 0
                max_jnode = dict()
                for bus_streamer in bus_streamers:
                    stat_jnode = bus_streamer.get('stat') or dict()
                    if stat_jnode.get('endpoints', 0) > max_endpoints:
                        max_jnode = stat_jnode

                return max_jnode.get('wh')

    def get_stats(self, term_pid: str) -> dict:
        term_uimod = Umod.create_term(term_pid)
        term_uuid = TerminalUiconf(term_uimod.conf_path).get_uuid()
        stream_stats = {
            'stream': f'api/v2/origins/media/{term_uuid}/live/stream'
        }

        cam_stream_wh = self.get_stream_wh()
        if cam_stream_wh:
            stream_stats['resolution'] = cam_stream_wh

        return {
            'media': [stream_stats]
        }


class CamUiStats:
    def __init__(self, intrinsic_jnode: dict | None) -> None:
        self._intrinsic_jnode = intrinsic_jnode

    @classmethod
    def create(cls, term_pid: str) -> Self:
        mv_cam_uimod = Umod(term_pid, UTYPE_MV_CAMERA, '0')
        rtsp_cam_uimod = Umod(term_pid, UTYPE_RTSP_CAMERA, '0')

        mv_jnode = mv_cam_uimod.load_conf(default=None) or dict()

        mv_intr = mv_jnode.get('ro', {}).get('intrinsic')
        if mv_intr:
            return cls(mv_intr)
        else:
            rtsp_jnode = rtsp_cam_uimod.load_conf(default=None) or dict()
            rtsp_intr = rtsp_jnode.get('ro', {}).get('intrinsic') \
                or rtsp_jnode.get('rw', {}).get('conf', {}).get('intrinsic')

            if rtsp_intr:
                return cls(rtsp_intr)

        return cls(None)

    def get_sensorwpx_sensorhpx_focalpx(self) -> tuple[float, float, float] | None:

        if self._intrinsic_jnode:
            sensor_w_px, sensor_h_px = self._intrinsic_jnode['frustum']['sensor-size']
            focus_dist_px = self._intrinsic_jnode.get('calibration', dict()).get('focal-dist')
            if focus_dist_px is None:
                focus_dist_mm = self._intrinsic_jnode['frustum']['focal-dist']
                pixel_size_um = self._intrinsic_jnode['frustum']['pixel-size']
                focus_dist_px = focus_dist_mm / pixel_size_um * 1000

            return sensor_w_px, sensor_h_px, focus_dist_px


def get_terminal_stats(term_pid: str) -> dict:
    root_uinfo = rootinfo.decode_info().root
    term_umod = Umod.create_term(term_pid)
    term_uuid = TerminalUiconf(term_umod.conf_path).get_uuid()
    video_wh = CamDgStats.create(term_pid).get_stream_wh()

    stats_jnode = {
        'media': []
    }

    term_uinfo = unet.find_unit_r(root_uinfo, uref=term_umod.ref)
    if term_uinfo is None:
        eventlog.log_d(f'fail: NotFoundError ({UTYPE_TERMINAL})')
        raise NotFoundError

    http_uinfo = unet.find_unit_r(term_uinfo, utype=UTYPE_HTTP_STREAMING)
    if http_uinfo is not None:
        http_umod = Umod.create(http_uinfo['ref'])
        http_uconf = http_umod.load_conf()
        http_uconf_pub = http_uconf['rw']
        http_enabled = http_uconf_pub.get('enabled', False)
        if http_enabled:
            http_stats = {
                'stream': f'api/v2/origins/media/{term_uuid}/live/stream'
            }

            if video_wh:
                http_stats['resolution'] = video_wh

            stats_jnode['media'].append(http_stats)

    rtsp_uinfo = unet.find_unit_r(term_uinfo, utype=UTYPE_RTSP_STREAMING)
    if rtsp_uinfo is not None:
        rtsp_umod = Umod.create(rtsp_uinfo['ref'])
        rtsp_uconf = rtsp_umod.load_conf()
        rtsp_uconf_pub = rtsp_uconf['rw']
        rtsp_enabled = rtsp_uconf_pub.get('enabled', False)
        if rtsp_enabled:
            rtsp_stats = {
                'stream': rtsp_uconf_pub.get('streaming', {}).get('url', 'try again later')  # TODO
            }

            stats_jnode['media'].append(rtsp_stats)

    return stats_jnode


def get_detector_limits_stats(
        term_pid: str,
        vehicle: bool = True,
        scene: bool = True,
        lanes: bool = False,
        classes: bool = False) -> dict:

    root_info = rootinfo.decode_info().root

    def store_factory(es_info: dict) -> dict:
        return {
            'id': es_info['ref'].split('.')[-1],  # TODO: for current ui
            'ref': es_info['ref'],
            'name': es_info.get('name')
        }

    term_uimod = Umod.create_term(term_pid)
    term_uuid = TerminalUiconf(term_uimod.conf_path).get_uuid()
    video_wh = CamDgStats.create(term_pid).get_stream_wh()
    video_w, video_h = (640, 480) if video_wh is None else video_wh

    stats_jnode = {
        'media': [{
            'resolution': [video_w, video_h],
            'stream': f'api/v2/origins/media/{term_uuid}/live/stream'
        }]
    }

    limits_jnode = {
        'images': dict(),
        'stores': unet.map_units_r(root_info, store_factory, utypes=[UTYPE_EVENTSTORE])
    }

    if vehicle:
        limits_jnode['images']['vehicle'] = {'max-width': 640}

    if scene:
        limits_jnode['images']['scene'] = {'max-width': video_w}

    if lanes:
        roadway_uimod = Umod(term_pid, UTYPE_ROADWAY, '0')  # TODO
        roadway_uiconf = roadway_uimod.load_conf()
        roadway_lanes = roadway_uiconf['rw']['args']['lanes']
        limits_jnode['lanes'] = [x['name'] for x in roadway_lanes]

    if classes:
        limits_jnode['classes'] = MMR_CLASSES

    return {
        'limits': limits_jnode,
        'stats': stats_jnode
    }
