from typing import Dict, List, Tuple, Optional

from edge import eventlog
from edge.services.types.plateinfo import IPlateInfoService
from edge.types.dir import Dir
from edge.types.jnode.imageref import decode__image_ref
from edge.types.jnode.track import decode__lr_track_item

from edge.types.shape.quad import decode__quadi, encode__quadi
from edge.types.shape.rect import decode__recti, encode__recti
from tracker.types.lr import LrVrp, LrSceneItem, LrScene, LrTrackSet, LrSceneStat, LrTrackStat, LrTrack


def encode__lr_scene_item(self: LrSceneItem) -> Dict:
    jnode = {
        'Id': self.oid,
        'Dir': self.dir,
        'Plate': {
            'P': self.vrp.text_ucode,
            'Q': encode__quadi(self.vrp.text_quad)
        },
        'Trace': self.trace
    }
    if self.vrp.vehicle_rect:
        jnode['Plate']['VR'] = encode__recti(self.vrp.vehicle_rect)

    return jnode


def decode__lr_scene_item__vrp(jnode: Dict) -> LrVrp:
    """
    :param jnode:
        LrScene#/Lr/:x/Plate
    """
    text_ucode = jnode['P']
    text_quad = decode__quadi(jnode['Q'])

    vehicle_rect = None
    vehicle_rect_jnode = jnode.get('VR')
    if vehicle_rect_jnode:
        vehicle_rect = decode__recti(vehicle_rect_jnode)

    return LrVrp(text_ucode, text_quad, vehicle_rect)


def decode__lr_scene_item(jnode: Dict, scene_ts: int) -> LrSceneItem:
    """
    :param jnode:
        LrScene#/Lr/:x
    :param scene_ts:
        LrScene#/Image/Ts
    """
    lr_oid = jnode['Id']
    lr_dir = jnode['Dir']
    lr_vrp = decode__lr_scene_item__vrp(jnode['Plate'])
    lr_trace: List[int] = jnode['Trace']

    return LrSceneItem(scene_ts, int(lr_oid), lr_vrp, Dir(lr_dir), lr_trace)


def decode__lr_scene(jnode: Dict) -> LrScene:
    """
    :param jnode:
        LrScene#/
    :raises ValueError:
    """
    sender: Tuple[int, int] = jnode['Source']
    stat = int(jnode['Stat'])

    imref_jnode = jnode.get('Image')
    if imref_jnode:
        imref = decode__image_ref(imref_jnode)
    else:
        eventlog.log_d('fail: image-jnode is absent')
        raise ValueError

    items_jnode: List[Dict] = jnode['Lr']
    items = [decode__lr_scene_item(item_jnode, imref.ts) for item_jnode in items_jnode]

    return LrScene(sender, LrSceneStat(stat), imref, items)


def decode__lr_track(jnode: Dict, plate_info_service: IPlateInfoService) -> LrTrackSet:
    """
    :param jnode: LrTrack#/
    :param plate_info_service: IPlateInfoService
    """
    alarms_jnode: List[Dict] | None = jnode.get('Alarms')
    wa_jnode: Optional[Dict] = jnode.get('Wa')
    lr_jnode = jnode['Lr']

    sender: Tuple[int, int] = lr_jnode['Source']
    oids: List[int] = lr_jnode['Id']

    plate_text = lr_jnode['Plate']['P']
    plate_template = plate_info_service.get_plate_template(lr_jnode['Plate']['T'])
    stat: int = lr_jnode['Stat']
    vrp_dir = Dir(lr_jnode['Dir'])

    only_detected_bit: bool = stat == 1
    with_lost_bit = bool(stat & 0x04)

    if only_detected_bit:
        status = LrTrackStat.DETECTED
    elif with_lost_bit:
        status = LrTrackStat.LOST
    else:
        status = LrTrackStat.UNKNOWN

    plate_trace = [decode__lr_track_item(item_jnode) for item_jnode in lr_jnode['PlateTrace']]

    lr_track = LrTrack(sender, oids, plate_text, plate_template, plate_trace, status, vrp_dir)
    return LrTrackSet(lr_track, lr_jnode, wa_jnode, alarms_jnode)
